local emberMenu
local showOffWindow

local default_info = {
    {
        image = "images/randomhint",
        Title = "Embercore news",
        description = "Welcome to Embercore! \n\nWe are a new server that is constantly evolving. We are always looking for ways to improve and expand our server. \nIf you have any questions or need help, feel free to ask in the help channel. \nWe hope you enjoy your time on Embercore!"
    },
}


function init()
    -- Load the UI file
    emberMenu = g_ui.displayUI('embermenu')

    -- Retrieve references to the windows and their components
    showOffWindow = emberMenu:recursiveGetChildById('showOffWindow')
    showOffWindow.title = showOffWindow:recursiveGetChildById('showOffWindowText')

    -- Populate the `showOffWindow` as usual
    local scrollable = showOffWindow:recursiveGetChildById('contentsPanel')
    local widget = g_ui.createWidget('ShowOffWidget', scrollable)
    local description = widget:recursiveGetChildById('description')
    local image = widget:recursiveGetChildById('image')

    -- Populate text and image for `showOffWindow`
    math.randomseed(os.time())
    local randomIndex = math.random(1, #default_info)
    local randomItem = default_info[randomIndex]
    showOffWindow.title:setText(tr(randomItem.Title))
    image:setImageSource(randomItem.image)
    description:setText(tr(randomItem.description))
    widget:resize(widget:getWidth(), description:getHeight())

    -- Connecting game events to handle visibility changes
    connect(g_game, {
        onGameStart = onGameStart,
        onGameEnd = onGameEnd
    })
    
    -- Hide the menu if the player is already logged in
    if g_game.isOnline() then
        hide()
    end
end

-- Terminate function to disconnect game events
function terminate()
    disconnect(g_game, {
        onGameStart = onGameStart,
        onGameEnd = onGameEnd
    })
    emberMenu:destroy()
end

-- Called when the game starts, hides the menu
function onGameStart() 
    hide()
end

-- Called when the game ends, shows the menu
function onGameEnd()
    show()
end

function hide()
    emberMenu:setVisible(false)
    emberMenu:lower()
    emberMenu:focus()  -- Ensure focus is removed from emberMenu
end

-- Show emberMenu
function show()
    emberMenu:setVisible(true)
    emberMenu:show()
    emberMenu:raise()
end


function setShowOffData(data)
    local widget = g_ui.createWidget('ShowOffWidget', showOffWindow)
    local image = widget:recursiveGetChildById('image')

    if data.image and data.image:sub(1, 4):lower() == "http" then
        HTTP.downloadImage(data.image, function(path, err)
            if err then
                g_logger.warning("HTTP error: " .. err .. " - " .. data.image)
                return
            end
            image:setImageSource(path)
        end)
    else
        image:setImage(data.image)
    end

    local description = widget:recursiveGetChildById('description')

    showOffWindow.title:setText(tr(data.title))
    description:setText(tr(data.description))
end

